#include "DS1307.h"
#include "util1.h"

#define RGB_LED_R   PORTB.F3
#define RGB_LED_G   PORTB.F4
#define RGB_LED_B   PORTB.F5

#define GRN_LED_1   PORTA.F4
#define GRN_LED_2   PORTA.F5
#define RED_LED_1   PORTE.F0
#define RED_LED_2   PORTE.F1

#define INPUT       1
#define OUTPUT      0

#define ON          1
#define OFF         0

#define ENABLE      1
#define DISABLE     0

#define TMR0_PRESET   94 // preset for timer register(2 cycle adjusted)

const char *dbg1_message[] = {"Chronulator 2.0 Init",   // 0
                              "Time: ",                 // 1
                              "Date: ",                 // 2
                              "Display mode",           // 3
                              "Set mode",               // 4
                              "Calibration mode",       // 5
                              "UP btn",                 // 6
                              "DOWN btn",               // 7
                              "DATE/TIME btn",          // 8
                              "SET btn",                // 9
                              "Hr/mth duty = ",         // 10
                              "Min/day duty = "         // 11
};

//  EEPROM,       
const char hr_duty_epr_adr[]  = {0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007, 0x0008, 0x0009, 0x000A, 0x000B, 0x000C, 0x000D, 0x000E, 0x000F,
                                 0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017, 0x0018
};
const char mn_duty_epr_adr[]  = {0x0020, 0x0021, 0x0022, 0x0023, 0x0024, 0x0025, 0x0026, 0x0027, 0x0028, 0x0029, 0x002A, 0x002B, 0x002C, 0x002D, 0x002E, 0x002F,
                                 0x0030, 0x0031, 0x0032, 0x0033, 0x0034, 0x0035, 0x0036, 0x0037, 0x0038, 0x0039, 0x003A, 0x003B, 0x003C, 0x003D, 0x003E, 0x003F,
                                 0x0040, 0x0041, 0x0042, 0x0043, 0x0044, 0x0045, 0x0046, 0x0047, 0x0048, 0x0049, 0x004A, 0x004B, 0x004C, 0x004D, 0x004E, 0x004F,
                                 0x0050, 0x0051, 0x0052, 0x0053, 0x0054, 0x0055, 0x0056, 0x0057, 0x0058, 0x0059, 0x005A, 0x005B, 0x005C
};
const char day_duty_epr_adr[] = {0x0060, 0x0061, 0x0062, 0x0063, 0x0064, 0x0065, 0x0066, 0x0067, 0x0068, 0x0069, 0x006A, 0x006B, 0x006C, 0x006D, 0x006E, 0x006F,
                                 0x0070, 0x0071, 0x0072, 0x0073, 0x0074, 0x0075, 0x0076, 0x0077, 0x0078, 0x0079, 0x007A, 0x007B, 0x007C, 0x007D, 0x007E, 0x007F
};
const char mth_duty_epr_adr[] = {0x0080, 0x0081, 0x0082, 0x0083, 0x0084, 0x0085, 0x0086, 0x0087, 0x0088, 0x0089, 0x008A, 0x008B, 0x008C
};

volatile unsigned int RTC = 0;   // Real time counter

// 
char DisplayMode          = 0;   // 0 -  , 1 -  

char PrevDisplayMode      = 1;   //   

char OperationMode        = 0;   // 0 -  /, 1 -  /, 2 - 

char SetMode              = 0;   // 0 -  , 1 -  , 2 -  ,
                                 // 3 -  , 4 -  , 5 -     DS1307
                                 
char CalibrationMode      = 0;   // 0 -  , 1 -   , 2 -  ,
                                 // 3 -  , 4 -  ,
                                 // 5 -  , 6 -     EEPROM
                                 
char CalibrationSelect    = 1;   // 1 -   , 2 -   , 3 -   ,
                                 // 4 -   

char BonusFlag            = 0;   // - (   6- )

char out_str1[24] = "";

char day  = 0;
char mth  = 0;
char year = 0;
char dow  = 0;
char hr   = 0;
char mn   = 0;
char sec  = 0;

char new_day  = 0;
char new_mth  = 0;
char new_year = 0;
char new_dow  = 0;
char new_hr   = 0;
char new_mn   = 0;
char new_sec  = 0;

char tmp[4];
char output[21];

char oldstate[4] = {0, 0, 0, 0};

char hoursDuty   = 0;
char minutesDuty = 0;
char daysDuty    = 0;
char monthsDuty  = 0;
char yearDuty1   = 0;
char yearDuty2   = 0;

char CurrentCalibrationDuty = 0;
char CurrentDutyElement     = 0;
char new_duty               = 0;

char GRN_LED_1_status = 0;
char GRN_LED_2_status = 0;
char RED_LED_1_status = 0;
char RED_LED_2_status = 0;

char RGB_LED_G_status = 0;
char RGB_LED_R_status = 0;
char RGB_LED_B_status = 0;

void Init_IO_Pins (void) {
 ANSEL     = 0b0000000;      //   -  /
 ANSELH    = 0b0000000;
 
 //    
 TRISA.RA0 = INPUT;          // UP
 TRISA.RA1 = INPUT;          // DOWN
 TRISA.RA2 = INPUT;          // DATE/CLOCK
 TRISA.RA3 = INPUT;          // SET
 
 //    RGB-
 TRISB.RB3 = OUTPUT;
 TRISB.RB4 = OUTPUT;
 TRISB.RB5 = OUTPUT;

 //  RGB-
 RGB_LED_R = OFF;
 RGB_LED_G = OFF;
 RGB_LED_B = OFF;

 //     
 TRISA.RA4 = OUTPUT;         // LED1 GREEN
 TRISA.RA5 = OUTPUT;         // LED2 GREEN
 TRISE.RA0 = OUTPUT;         // LED1 RED
 TRISE.RA1 = OUTPUT;         // LED2 RED

 //  
 GRN_LED_1 = OFF;
 GRN_LED_2 = OFF;
 RED_LED_1 = OFF;
 RED_LED_2 = OFF;
}

//    TMR0 (  PIC Timer Calculator)
void TMR0_setup(void) {
 // Timer0 Registers: Prescaler=1:32; TMR0 Preset=94; Freq=976,5625Hz; Period=1024,00 s
 OPTION_REG.T0CS = 0;  // bit 5 TMR0 Clock Source Select bit:0=Internal Clock (CLKO) / 1=Transition on T0CKI pin
 OPTION_REG.T0SE = 0;  // bit 4 TMR0 Source Edge Select bit: 0=low/high / 1=high/low
 OPTION_REG.PSA  = 0;  // bit 3 Prescaler Assignment bit: 0=Prescaler is assigned to the Timer0
 OPTION_REG.PS2  = 1;  // bits 2-0  PS2:PS0: Prescaler Rate Select bits
 OPTION_REG.PS1  = 0;
 OPTION_REG.PS0  = 0;
 TMR0            = TMR0_PRESET;   // preset for timer register(2 cycle adjusted)
 INTCON.T0IE     = ENABLE;        //    TMR0
}

void UpdateHoursDuty (void) {
 PWM2_Set_Duty(hoursDuty);
}

void UpdateMinutesDuty (void) {
 PWM1_Set_Duty(minutesDuty);
}

void UpdateMonthsDuty (void) {
 PWM2_Set_Duty(monthsDuty);
}

void UpdateDaysDuty (void) {
 PWM1_Set_Duty(daysDuty);
}

void UpdateYearDuty1 (void) {
 PWM2_Set_Duty(yearDuty1);
}

void UpdateYearDuty2 (void) {
 PWM1_Set_Duty(yearDuty2);
}

void SetDutyHours (char newDuty) {
 if (hoursDuty != newDuty) {
    hoursDuty = newDuty;
    UpdateHoursDuty();
 }
}

void SetDutyMinutes (char newDuty) {
 if (minutesDuty != newDuty) {
    minutesDuty = newDuty;
    UpdateMinutesDuty();
 }
}

void SetDutyMonths (char newDuty) {
 if (monthsDuty != newDuty) {
    monthsDuty = newDuty;
    UpdateMonthsDuty();
 }
}

void SetDutyDays (char newDuty) {
 if (daysDuty != newDuty) {
    daysDuty = newDuty;
    UpdateDaysDuty();
 }
}

void SetDutyYear1 (char newDuty) {
 if (yearDuty1 != newDuty) {
    yearDuty1 = newDuty;
    UpdateYearDuty1();
 }
}

void SetDutyYear2 (char newDuty) {
 if (yearDuty2 != newDuty) {
    yearDuty2 = newDuty;
    UpdateYearDuty2();
 }
}

void ButtonHandler (void) {
 if (Button(&PORTA, 0, 10, 1)) oldstate[0] = 1;  // UP button pressed
 if (Button(&PORTA, 1, 10, 1)) oldstate[1] = 1;  // DOWN button pressed
 if (Button(&PORTA, 2, 10, 1)) oldstate[2] = 1;  // DATE/TIME button pressed
 if (Button(&PORTA, 3, 10, 1)) oldstate[3] = 1;  // SET button pressed

 if (Button(&PORTA, 0, 10, 0) && oldstate[0]) {  // UP button released
    switch (OperationMode) {
     case 0:
            break;
     case 1:
            switch (SetMode) {
             case 0:
                    break;
             case 1:
                    if (new_hr < 24) {
                       new_hr++;
                    }
                    SetDutyHours(EEPROM_Read(hr_duty_epr_adr[new_hr]));
                    break;
             case 2:
                    if (new_mn < 60) {
                       new_mn++;
                    }
                    SetDutyMinutes(EEPROM_Read(mn_duty_epr_adr[new_mn]));
                    break;
             case 3:
                    if (new_mth < 12) {
                       new_mth++;
                    }
                    SetDutyMonths(EEPROM_Read(mth_duty_epr_adr[new_mth]));
                    break;
             case 4:
                    if (new_day < 31) {
                       new_day++;
                    }
                    SetDutyDays(EEPROM_Read(day_duty_epr_adr[new_day]));
                    break;
             case 5:
                    if (new_year < 31) {
                       new_year++;
                    }
                    SetDutyYear2(EEPROM_Read(day_duty_epr_adr[new_year]));
                    break;
            }
            break;
     case 2:
            if (new_duty < 255) {
               new_duty++;
               switch (CalibrationMode) {
                case 2:
                       SetDutyHours(new_duty);
                       break;
                case 3:
                       SetDutyMinutes(new_duty);
                       ByteToStr(mn, tmp);
                       Usart_Write_Str (tmp);
                       Usart_Write_Str (" - ");
                       break;
                case 4:
                       SetDutyMonths(new_duty);
                       break;
                case 5:
                       SetDutyDays(new_duty);
                       break;
               }
               ByteToStr(new_duty, tmp);
               Usart_WriteLn_Str (tmp);
            }
            break;
    }
    strConstCopy (out_str1, dbg1_message[6]);       // "UP btn"
    Usart_WriteLn_Str(out_str1);
    oldstate[0] = 0;
 }
 else if (Button(&PORTA, 1, 10, 0) && oldstate[1]) { // DOWN button released
    switch (OperationMode) {
     case 0:
            break;
     case 1:
            switch (SetMode) {
             case 0:
                    break;
             case 1:
                    if (new_hr > 0) {
                       new_hr--;
                    }
                    SetDutyHours(EEPROM_Read(hr_duty_epr_adr[new_hr]));
                    break;
             case 2:
                    if (new_mn > 0) {
                       new_mn--;
                    }
                    SetDutyMinutes(EEPROM_Read(mn_duty_epr_adr[new_mn]));
                    break;
             case 3:
                    if (new_mth > 1) {
                       new_mth--;
                    }
                    SetDutyMonths(EEPROM_Read(mth_duty_epr_adr[new_mth]));
                    break;
             case 4:
                    if (new_day > 1) {
                       new_day--;
                    }
                    SetDutyDays(EEPROM_Read(day_duty_epr_adr[new_day]));
                    break;
             case 5:
                    if (new_year > 1) {
                       new_year--;
                    }
                    SetDutyYear2(EEPROM_Read(day_duty_epr_adr[new_year]));
                    break;
            }
            break;
     case 2:
            if (new_duty > 0) {
               new_duty--;
               switch (CalibrationMode) {
                case 2:
                       SetDutyHours(new_duty);
                       break;
                case 3:
                       SetDutyMinutes(new_duty);
                       ByteToStr(mn, tmp);
                       Usart_Write_Str (tmp);
                       Usart_Write_Str (" - ");
                       break;
                case 4:
                       SetDutyMonths(new_duty);
                       break;
                case 5:
                       SetDutyDays(new_duty);
                       break;
               }
               ByteToStr(new_duty, tmp);
               Usart_WriteLn_Str (tmp);
            }
            break;
    }
    strConstCopy (out_str1, dbg1_message[7]);       // "DOWN btn"
    Usart_WriteLn_Str(out_str1);
    oldstate[1] = 0;
 }
 else if (Button(&PORTA, 2, 10, 0) && oldstate[2]) { // DATE/TIME button released
  switch (OperationMode) {
   case 0:
          switch (DisplayMode) {
           case 0:
                  DisplayMode = 1;
                  UpdateMonthsDuty();
                  UpdateDaysDuty();
                  break;
           case 1:
                  DisplayMode = 0;
                  UpdateHoursDuty();
                  UpdateMinutesDuty();
                  break;
          }
          break;
   case 1:
          break;
   case 2:
          if (CalibrationMode == 1) {
             CalibrationSelect++;
             if (CalibrationSelect == 5) {
                CalibrationSelect = 1;
             }
             switch (CalibrationSelect) {
              case 1:
                     GRN_LED_1 = ON;
                     GRN_LED_2 = OFF;
                     RED_LED_1 = OFF;
                     RED_LED_2 = OFF;
                     break;
              case 2:
                     GRN_LED_1 = OFF;
                     GRN_LED_2 = ON;
                     RED_LED_1 = OFF;
                     RED_LED_2 = OFF;
                     break;
              case 3:
                     GRN_LED_1 = OFF;
                     GRN_LED_2 = OFF;
                     RED_LED_1 = ON;
                     RED_LED_2 = OFF;
                     break;
              case 4:
                     GRN_LED_1 = OFF;
                     GRN_LED_2 = OFF;
                     RED_LED_1 = OFF;
                     RED_LED_2 = ON;
                     break;
             }
          }
          break;
  }
  strConstCopy (out_str1, dbg1_message[8]);       // "DATE/TIME btn"
  Usart_WriteLn_Str(out_str1);
  oldstate[2] = 0;
 }
 else if (Button(&PORTA, 3, 10, 0) && oldstate[3]) { // SET button released
  switch (OperationMode) {
   case 0:
          OperationMode = 1;
          SetMode = 1;
          new_hr = hr;
          new_mn = mn;
          new_mth = mth;
          new_day = day;
          new_year = year;
          RGB_LED_G = ON;
          RGB_LED_R = OFF;
          GRN_LED_1 = ON;
          GRN_LED_2 = ON;
          RED_LED_1 = OFF;
          RED_LED_2 = OFF;
          UpdateHoursDuty();
          UpdateMinutesDuty();
          strConstCopy (out_str1, dbg1_message[4]);  // "Set mode"
          Usart_WriteLn_Str(out_str1);
          break;
   case 1:
          SetMode++;
          switch (SetMode) {
           case 0:
                  break;
           case 1:
           case 2:
                  RGB_LED_G = ON;
                  RGB_LED_R = OFF;
                  GRN_LED_1 = ON;
                  GRN_LED_2 = ON;
                  RED_LED_1 = OFF;
                  RED_LED_2 = OFF;
                  UpdateHoursDuty();
                  UpdateMinutesDuty();
                  break;
           case 3:
           case 4:
                  RGB_LED_G = OFF;
                  RGB_LED_R = ON;
                  GRN_LED_1 = OFF;
                  GRN_LED_2 = OFF;
                  RED_LED_1 = ON;
                  RED_LED_2 = ON;
                  UpdateMonthsDuty();
                  UpdateDaysDuty();
                  break;
           case 5:
                  RGB_LED_G = OFF;
                  RGB_LED_R = ON;
                  GRN_LED_1 = OFF;
                  GRN_LED_2 = OFF;
                  RED_LED_1 = ON;
                  RED_LED_2 = ON;
                  RED_LED_1_status = 0;
                  RED_LED_2_status = 0;
                  UpdateYearDuty1();
                  UpdateYearDuty2();
                  break;
           case 6:
                  if (new_hr != hr || new_mn != mn) {
                     ds1307_set_time(new_hr, new_mn, sec);
                  }
                  if (new_day != day || new_mth != mth || new_year != year) {
                     ds1307_set_date(new_day, new_mth, new_year);
                  }
                  SetMode = 0;
                  OperationMode = 0;
                  DisplayMode = 0;
                  PrevDisplayMode = 1;
                  UpdateHoursDuty();
                  UpdateMinutesDuty();
                  strConstCopy (out_str1, dbg1_message[3]);// "Display mode"
                  Usart_WriteLn_Str(out_str1);
                  break;
          }
          break;
   case 2:
          switch (CalibrationMode) {
           case 0:
                  break;
           case 1:
                  switch (CalibrationSelect) {
                   case 1:
                          hr = 0;
                          CurrentCalibrationDuty = EEPROM_Read(hr_duty_epr_adr[hr]);
                          new_duty = CurrentCalibrationDuty;
                          SetDutyHours(CurrentCalibrationDuty);
                          CalibrationMode = 2;
                          break;
                   case 2:
                          mn = 0;
                          CurrentCalibrationDuty = EEPROM_Read(mn_duty_epr_adr[mn]);
                          new_duty = CurrentCalibrationDuty;
                          SetDutyMinutes(CurrentCalibrationDuty);
                          CalibrationMode = 3;
                          break;
                   case 3:
                          mth = 1;
                          CurrentCalibrationDuty = EEPROM_Read(mth_duty_epr_adr[mth]);
                          new_duty = CurrentCalibrationDuty;
                          SetDutyMonths(CurrentCalibrationDuty);
                          CalibrationMode = 4;
                          break;
                   case 4:
                          day = 1;
                          CurrentCalibrationDuty = EEPROM_Read(day_duty_epr_adr[day]);
                          new_duty = CurrentCalibrationDuty;
                          SetDutyDays(CurrentCalibrationDuty);
                          CalibrationMode = 5;
                          break;
                  }
                  break;
           case 2:
                  if (new_duty != CurrentCalibrationDuty) {
                     EEPROM_Write(hr_duty_epr_adr[hr], new_duty);
                     Delay_ms(25);
                  }
                  hr++;
                  if (hr > 24) {
                     hr = 0;
                     CalibrationMode = 1;
                     CalibrationSelect = 2;
                     SetDutyHours(0);
                     GRN_LED_1 = OFF;
                     GRN_LED_2 = ON;
                     RED_LED_1 = OFF;
                     RED_LED_2 = OFF;
                     RGB_LED_G = OFF;
                     RGB_LED_R = OFF;
                  } else {
                    CurrentCalibrationDuty = EEPROM_Read(hr_duty_epr_adr[hr]);
                    new_duty = CurrentCalibrationDuty;
                    SetDutyHours(CurrentCalibrationDuty);
                  }
                  break;
           case 3:
                  if (new_duty != CurrentCalibrationDuty) {
                     EEPROM_Write(mn_duty_epr_adr[mn], new_duty);
                     Delay_ms(25);
                  }
                  mn++;
                  if (mn > 60) {
                     mn = 0;
                     CalibrationMode = 1;
                     CalibrationSelect = 3;
                     SetDutyMinutes(0);
                     GRN_LED_1 = OFF;
                     GRN_LED_2 = OFF;
                     RED_LED_1 = ON;
                     RED_LED_2 = OFF;
                     RGB_LED_G = OFF;
                     RGB_LED_R = OFF;
                  } else {
                    CurrentCalibrationDuty = EEPROM_Read(mn_duty_epr_adr[mn]);
                    new_duty = CurrentCalibrationDuty;
                    SetDutyMinutes(CurrentCalibrationDuty);
                  }
                  break;
           case 4:
                  if (new_duty != CurrentCalibrationDuty) {
                     EEPROM_Write(mth_duty_epr_adr[mth], new_duty);
                     Delay_ms(25);
                  }
                  mth++;
                  if (mth > 12) {
                     mth = 0;
                     CalibrationMode = 1;
                     CalibrationSelect = 4;
                     SetDutyMonths(0);
                     GRN_LED_1 = OFF;
                     GRN_LED_2 = OFF;
                     RED_LED_1 = OFF;
                     RED_LED_2 = ON;
                     RGB_LED_G = OFF;
                     RGB_LED_R = OFF;
                  } else {
                    CurrentCalibrationDuty = EEPROM_Read(mth_duty_epr_adr[mth]);
                    new_duty = CurrentCalibrationDuty;
                    SetDutyMonths(CurrentCalibrationDuty);
                  }
                  break;
           case 5:
                  if (new_duty != CurrentCalibrationDuty) {
                     EEPROM_Write(day_duty_epr_adr[day], new_duty);
                     Delay_ms(25);
                  }
                  day++;
                  if (day > 31) {
                     day = 0;
                     CalibrationMode = 1;
                     CalibrationSelect = 1;
                     SetDutyDays(0);
                     GRN_LED_1 = ON;
                     GRN_LED_2 = OFF;
                     RED_LED_1 = OFF;
                     RED_LED_2 = OFF;
                     RGB_LED_G = OFF;
                     RGB_LED_R = OFF;
                  } else {
                    CurrentCalibrationDuty = EEPROM_Read(day_duty_epr_adr[day]);
                    new_duty = CurrentCalibrationDuty;
                    SetDutyDays(CurrentCalibrationDuty);
                  }
                  break;
          }
          break;
  }
  strConstCopy (out_str1, dbg1_message[9]);      // "SET btn"
  Usart_WriteLn_Str(out_str1);
  oldstate[3] = 0;
 }
}

void TimeAndDateUsartWrite (void) {
 strConstCopy (out_str1, dbg1_message[1]);       // "Time: "
 Usart_Write_Str(out_str1);
 ByteToStr(hr, tmp);
 Usart_Write_Str (tmp);
 Usart_Write_Str (":");
 ByteToStr(mn, tmp);
 Usart_Write_Str (tmp);
 Usart_Write_Str (":");
 ByteToStr(sec, tmp);
 Usart_Write_Str (tmp);
 Usart_Write_Str (" ");

 strConstCopy (out_str1, dbg1_message[10]);      // "Hr/mth duty = "
 Usart_Write_Str(out_str1);
 ByteToStr(hoursDuty, tmp);
 Usart_Write_Str(tmp);
 Usart_Write_Str (" ");
 strConstCopy (out_str1, dbg1_message[11]);      // "Min/day duty = "
 Usart_Write_Str(out_str1);
 ByteToStr(minutesDuty, tmp);
 Usart_WriteLn_Str (tmp);
     
 strConstCopy (out_str1, dbg1_message[2]);       // "Date: "
 Usart_Write_Str(out_str1);
 ByteToStr(day, tmp);
 Usart_Write_Str (tmp);
 Usart_Write_Str ("-");
 ByteToStr(mth, tmp);
 Usart_Write_Str (tmp);
 Usart_Write_Str ("-");
 ByteToStr(year, tmp);
 Usart_Write_Str (tmp);
 Usart_Write_Str (" ");

 strConstCopy (out_str1, dbg1_message[10]);      // "Hr/mth duty = "
 Usart_Write_Str(out_str1);
 ByteToStr(monthsDuty, tmp);
 Usart_Write_Str(tmp);
 Usart_Write_Str (" ");
 strConstCopy (out_str1, dbg1_message[11]);      // "Min/day duty = "
 Usart_Write_Str(out_str1);
 ByteToStr(daysDuty, tmp);
 Usart_WriteLn_Str (tmp);
}

void interrupt (void) {
if (INTCON.T0IF) {                              //     TMR0
    RTC++;
    if ((RTC % 512) == 0) {                     //  512
       switch (OperationMode) {
        case 0:
               break;
        case 1:
               switch (SetMode) {
                case 0:
                       break;
                case 1:
                       GRN_LED_1_status = ~GRN_LED_1_status;
                       GRN_LED_1 = GRN_LED_1_status;
                       break;
                case 2:
                       GRN_LED_2_status = ~GRN_LED_2_status;
                       GRN_LED_2 = GRN_LED_2_status;
                       break;
                case 3:
                       RED_LED_1_status = ~RED_LED_1_status;
                       RED_LED_1 = RED_LED_1_status;
                       break;
                case 4:
                       RED_LED_2_status = ~RED_LED_2_status;
                       RED_LED_2 = RED_LED_2_status;
                       break;
                case 5:
                       RED_LED_1_status = ~RED_LED_1_status;
                       RED_LED_1 = RED_LED_1_status;
                       RED_LED_2_status = ~RED_LED_2_status;
                       RED_LED_2 = RED_LED_2_status;
                       break;
                case 6:
                       break;
               }
               break;
        case 2:
               switch (CalibrationMode) {
                case 0:
                       break;
                case 1:
                       RGB_LED_B_status = ~RGB_LED_B_status;
                       RGB_LED_B = RGB_LED_B_status;
                       break;
                case 2:
                case 3:
                       RGB_LED_B_status = ~RGB_LED_B_status;
                       RGB_LED_B = RGB_LED_B_status;
                       RGB_LED_G_status = ~RGB_LED_B_status;
                       RGB_LED_G = RGB_LED_G_status;
                       break;
                case 4:
                case 5:
                       RGB_LED_B_status = ~RGB_LED_B_status;
                       RGB_LED_B = RGB_LED_B_status;
                       RGB_LED_R_status = ~RGB_LED_B_status;
                       RGB_LED_R = RGB_LED_R_status;
                       break;
                case 6:
                       break;
               }
               break;
       }
    }
    INTCON.T0IF = 0;                             //   
    TMR0 = TMR0_PRESET;                          //     
 }
}

void main (void) {
 Init_IO_Pins();                                 //   /
 TMR0_setup();                                   //   TMR0
 
 INTCON.GIE = ENABLE;                            //  
 
 UART1_Init(9600);                               //   UART   9600 
 
 strConstCopy (out_str1, dbg1_message[0]);       // "Chronulator 2.0 Init"
 Usart_WriteLn_Str(out_str1);

 PWM1_Init(3000);                                //    
 PWM1_Start();
 PWM1_Set_Duty(0);

 PWM2_Init(3000);                                //    
 PWM2_Start();
 PWM2_Set_Duty(0);
 
 if (Button(&PORTA, 2, 100, 0)) {                //  DATE/TIME      100 ?
    OperationMode   = 2;                         //    
    CalibrationMode = 1;                         //     
    RGB_LED_B       = ON;                        //   
    GRN_LED_1       = ON;                        //       
    GRN_LED_2       = OFF;
    RED_LED_1       = OFF;
    RED_LED_2       = OFF;
    strConstCopy (out_str1, dbg1_message[5]);    // "Calibration mode"
    Usart_WriteLn_Str(out_str1);
 } else {
    OperationMode   = 0;                         //    /
    strConstCopy (out_str1, dbg1_message[3]);    // "Display mode"
    Usart_WriteLn_Str(out_str1);
 }

 I2C1_Init(100000);                              //   I2C
 ds1307_init();                                  //     DS1307
 
 if (OperationMode == 0) {
    //       DS1307      
    ds1307_get_time (&hr, &mn, &sec);
    ds1307_get_date (&day, &mth, &year, &dow);
    SetDutyYear1(EEPROM_Read(hr_duty_epr_adr[20]));
    Delay_ms(25);
    SetDutyYear2(EEPROM_Read(day_duty_epr_adr[year]));
    Delay_ms(25);
    SetDutyMonths(EEPROM_Read(mth_duty_epr_adr[mth]));
    Delay_ms(25);
    SetDutyDays(EEPROM_Read(day_duty_epr_adr[day]));
    Delay_ms(25);
    SetDutyHours(EEPROM_Read(hr_duty_epr_adr[hr]));
    Delay_ms(25);
    SetDutyMinutes(EEPROM_Read(mn_duty_epr_adr[mn]));
    Delay_ms(25);
 } else {
   //         
   SetDutyHours(0);
   SetDutyMinutes(0);
   SetDutyMonths(0);
   SetDutyDays(0);
 }

 
 while (1) {
  ButtonHandler();                               //   
  
  switch (OperationMode) {
   case 0:                                       //  /
          ds1307_get_time (&hr, &mn, &sec);
          ds1307_get_date (&day, &mth, &year, &dow);
          if (day == 6 && mth == 9) {
             RGB_LED_B = ON;
          } else {
            RGB_LED_B = OFF;
          }
          switch (DisplayMode) {
           case 0:                               // 
                  if (PrevDisplayMode == 1) {
                     RGB_LED_G = ON;
                     RGB_LED_R = OFF;
                     GRN_LED_1 = ON;
                     GRN_LED_2 = ON;
                     RED_LED_1 = OFF;
                     RED_LED_2 = OFF;
                     PrevDisplayMode = 0;
                  }
                  SetDutyHours(EEPROM_Read(hr_duty_epr_adr[hr]));
                  SetDutyMinutes(EEPROM_Read(mn_duty_epr_adr[mn]));
                  break;
           case 1:                               // 
                  if (PrevDisplayMode == 0) {
                     RGB_LED_G = OFF;
                     RGB_LED_R = ON;
                     GRN_LED_1 = OFF;
                     GRN_LED_2 = OFF;
                     RED_LED_1 = ON;
                     RED_LED_2 = ON;
                     PrevDisplayMode = 1;
                  }
                  SetDutyMonths(EEPROM_Read(mth_duty_epr_adr[mth]));
                  SetDutyDays(EEPROM_Read(day_duty_epr_adr[day]));
                  SetDutyYear1(EEPROM_Read(hr_duty_epr_adr[20]));
                  SetDutyYear2(EEPROM_Read(day_duty_epr_adr[year]));
                  break;
          }
          break;
   case 1:                                       //  /
          break;
   case 2:                                       // 
          break;
  }
  
  if (RTC > 1000) {
     //if (OperationMode == 1) {
        TimeAndDateUsartWrite();
     //}
     RTC = 0;
  }
 }
}